@@ -25,7 +25,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: combroyden2.c 1448 2008-10-21 03:07:28Z beerli $
+$Id: combroyden2.c 1574 2009-08-29 14:07:28Z beerli $
  
 -------------------------------------------------------*/
 
@@ -615,7 +615,7 @@ print_reset_big_param (FILE * file, MYREAL average, MYREAL maxtheta, long pop)
     if (!done)
     {
         done = TRUE;
-        memset (&checkparam, 0, sizeof (boolean) * 1000);
+        memset (&checkparam, 0, sizeof (boolean) * MAXPOP);
     }
     if (!checkparam[pop])
     {
