@@ -18,7 +18,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: fst.c 1323 2008-07-25 19:13:48Z beerli $
+$Id: fst.c 1348 2008-09-11 14:42:13Z beerli $
  
 -------------------------------------------------------*/
 /*! \file fst.c
@@ -235,7 +235,7 @@ calc_seq_fw (data_fmt * data, long numpop, long locus, MYREAL *fw)
     {
         fw[pop] = 0;
         nn =
-            data->seq->sites[locus] * (data->numind[pop][locus] *
+            data->seq[0]->sites[locus] * (data->numind[pop][locus] *
                                        data->numind[pop][locus] -
                                        data->numind[pop][locus]) / 2.;
         for (i = 0; i < data->numind[pop][locus]; i++)
@@ -243,7 +243,7 @@ calc_seq_fw (data_fmt * data, long numpop, long locus, MYREAL *fw)
             for (k = i + 1; k < data->numind[pop][locus]; k++)
             {
                 diff = 0.;
-                for (j = 0; j < data->seq->sites[locus]; j++)
+                for (j = 0; j < data->seq[0]->sites[locus]; j++)
                 {
                     diff +=
                         (data->yy[pop][i][locus][0][j] !=
@@ -270,14 +270,14 @@ calc_seq_fb (data_fmt * data, long numpop, long locus, MYREAL *fb)
         {
             temp = 0.;
             nn =
-                (MYREAL) data->seq->sites[locus] * data->numind[p1][locus] *
+                (MYREAL) data->seq[0]->sites[locus] * data->numind[p1][locus] *
                 data->numind[p2][locus];
             for (i = 0; i < data->numind[p1][locus]; i++)
             {
                 for (k = 0; k < data->numind[p2][locus]; k++)
                 {
                     diff = 0.;
-                    for (j = 0; j < data->seq->sites[locus]; j++)
+                    for (j = 0; j < data->seq[0]->sites[locus]; j++)
                     {
                         diff +=
                             (data->yy[p1][i][locus][0][j] !=
