@@ -11,7 +11,7 @@ This software is distributed free of charge for non-commercial use
 and is copyrighted. Of course, we do not guarantee that the software
 works and are not responsible for any damage you may cause or have.
  
-$Id: heating.c 1585 2009-10-28 22:55:15Z beerli $
+$Id: heating.c 1701 2010-06-13 21:00:45Z beerli $
  
 */
 /*! \file heating.c 
@@ -212,10 +212,13 @@ void thread_tree_update (void *thisworld)
 {
     long i;
     world_fmt * world = (world_fmt *) thisworld;
-    for (i = 0; i < world->options->heating_interval; i++)
-    {
+    // the version commented out would increase the runtime by the interval
+    // using the heating interval in heated_swap() is not as efficient in thread but would
+    // do the right thing, but just not checking that often.
+    //    for (i = 0; i < world->options->heating_interval; i++)
+    //{
 		run_one_update(thisworld);
-    }
+		//  }
 }
 
 ///
@@ -395,7 +398,7 @@ void adjust_temperatures_bounded(world_fmt ** universe, long hchains, long step,
             {
 	      universe[i]->averageheat += HEATCHECKINTERVAL * (1./universe[i]->heat - universe[i]->averageheat) / step;
 #ifdef DEBUG
-	      fprintf(stdout,"%i> chain %li: %f %f %li (step=%li (%f))\n", myID, i, 1./universe[i]->heat,universe[i]->averageheat, universe[i]->treeswapcount, step, (MYREAL) HEATCHECKINTERVAL/step);
+	      fprintf(stdout,"%i> chain %li: %f %f %li (step=%li (%f))\n", myID, i+1, 1./universe[i]->heat,universe[i]->averageheat, universe[i]->treeswapcount, step, (MYREAL) HEATCHECKINTERVAL/step);
 #endif
                 delta[i-1] = (MYREAL) universe[i]->treeswapcount;
 		if(delta[i-1] < 1.0)
