@@ -18,7 +18,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc2.c 1563 2009-07-30 18:28:33Z beerli $
+$Id: mcmc2.c 1573 2009-08-29 14:01:49Z beerli $
 -------------------------------------------------------*/
 /* \file mcmc2.c
 Contains low-level routines to manipulate the treechanger
@@ -456,12 +456,14 @@ target_oback (proposal_fmt * proposal)
     bid2 = proposal->osister->bid;
     v2   = proposal->vs;
     prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+    evaluate_beagle_instances_proposal (proposal,obb, proposal->oback, id0, proposal->oback->bid,
+					fabs (proposal->time - obb->tyme));
 #else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->osister->x, proposal->osister->scale, proposal->vs);
-#endif
     pseudoevaluate (proposal, proposal->xf, proposal->mf, obb, proposal->oback,
                     fabs (proposal->time - obb->tyme));
+#endif
 }
 
 void
@@ -513,20 +515,24 @@ target_ocousin (proposal_fmt * proposal)
     proposal->vs = obb->tyme - proposal->osister->tyme;
 #ifdef BEAGLE
     id0  = obb->id;
-    id1  = proposal->oback->id;
+    id1  = new_id(proposal->oback->id,proposal->world->sumtips);
     bid1 = proposal->oback->bid;
     v1   = proposal->v;
     id2  = proposal->osister->id;
     bid2 = proposal->osister->bid;
     v2   = proposal->vs;
     prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+    evaluate_beagle_instances_proposal (proposal,
+					showtop (crawlback (obb)), obb, obb->id, obb->bid,
+					(showtop (crawlback (obb))->tyme - obb->tyme));
+
 #else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->osister->x, proposal->osister->scale, proposal->vs);
-#endif
     pseudoevaluate (proposal, proposal->xf, proposal->mf,
                     showtop (crawlback (obb)), obb,
                     (showtop (crawlback (obb))->tyme - obb->tyme));
+#endif
 }
 
 void
@@ -562,14 +568,17 @@ target_rbcoa (proposal_fmt * proposal)
     bid2 = proposal->ocousin->bid;
     v2   = proposal->v;
     prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+    evaluate_beagle_instances_proposal(proposal, 
+				       showtop (crawlback (obb)), obb, obb->id, obb->bid,
+				       showtop (crawlback (obb))->tyme - obb->tyme);
 #else
         pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs,
                       proposal->ocousin->x, proposal->ocousin->scale,
                       proposal->v);
-#endif
         pseudoevaluate (proposal, proposal->xt, proposal->mt,
                         showtop (crawlback (obb)), obb,
                         showtop (crawlback (obb))->tyme - obb->tyme);
+#endif
         proposal->v = proposal->time - proposal->origin->tyme;
         proposal->vs = proposal->time - proposal->target->tyme;
         copy_x (proposal, proposal->xf, proposal->origin->x);
@@ -586,7 +595,7 @@ target_rbcoa (proposal_fmt * proposal)
     id1  = proposal->origin->id;
     bid1 = proposal->origin->bid;
     v1   = proposal->v;
-    id2  = proposal->target->id;
+    id2  = new_id(proposal->target->id, proposal->world->sumtips);//target == root->next->back
     bid2 = proposal->target->bid;
     v2   = proposal->vs;
     prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
@@ -627,7 +636,7 @@ target_obbcoa (proposal_fmt * proposal)
     long id0, id1, id2;
     long bid1, bid2;
     double v1, v2;
-    id0  = proposal->oback->id;
+    id0  = obb->id;
     id1  = proposal->osister->id;
     bid1 = proposal->osister->bid;
     v1   = proposal->vs;
@@ -656,18 +665,24 @@ target_obbcoa (proposal_fmt * proposal)
     id1  = proposal->origin->id;
     bid1 = proposal->origin->bid;
     v1   = proposal->v;
-    id2  = obb->id;
+    id2  = new_id(obb->id, proposal->world->sumtips);
     bid2 = obb->bid;
     v2   = proposal->vs;
     prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+    proposal->v = fabs (showtop (crawlback (obb))->tyme - proposal->time);
+    obbb = showtop (crawlback (obb));
+    /*check this carefully: fakes obb and uses id of oback which is inserted between obb and obbb*/
+    /*this should guarantee the proper evaluation but feeding the right ids into beagle*/
+    evaluate_beagle_instances_proposal (proposal,obbb, obb, proposal->oback->id, proposal->oback->bid,
+					proposal->v);
 #else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
-#endif
         proposal->v = fabs (showtop (crawlback (obb))->tyme - proposal->time);
         obbb = showtop (crawlback (obb));
-        pseudoevaluate (proposal, proposal->xf, proposal->mf, obbb, obb,
+        pseudoevaluate (proposal, proposal->xf, proposal->mf, obbb, /* is this correct??????*/ obb,
                         proposal->v);
+#endif
     }
     else
     {
@@ -718,9 +733,13 @@ target_stancoa (proposal_fmt * proposal)
 
                 memcpy (proposal->mt, proposal->osister->scale,
                         sizeof (MYREAL) * proposal->endsite);
-                /*       if (obb!=proposal->connect) { */
-                uppern = t_s_obbncon (proposal, obb);
-                /*} */
+		if (obb!=proposal->connect) { 
+		  uppern = t_s_obbncon (proposal, obb);
+                }
+		else
+		  {
+		    uppern = proposal->connect;
+		  }
                 proposal->v = proposal->time - proposal->origin->tyme;
                 copy_x (proposal, proposal->xf, proposal->origin->x);
 #ifdef UEP
@@ -733,7 +752,8 @@ target_stancoa (proposal_fmt * proposal)
                         sizeof (MYREAL) * proposal->endsite);
                 if (proposal->target != proposal->connect)
                 {
-                    t_s_tncon (proposal, obb);
+		  //                    t_s_tncon (proposal, obb);
+		  t_s_tncon (proposal, uppern);
                 }
                 else
                 {
@@ -789,10 +809,13 @@ target_stancoa (proposal_fmt * proposal)
         {
             children (proposalack, &d1, &d2);
             if (d1 == proposal->target)
-                d1 = d2;
+	      {
+		d1 = d2;
+		d2 = proposal->target;
+	      }
 #ifdef BEAGLE
 	    id0  = proposalack->id;
-	    id1  = proposal->oback->id;
+	    id1  = new_id(proposal->oback->id,proposal->world->sumtips);
 	    bid1 = proposal->oback->bid;
 	    v1   = proposal->v;
 	    id2  = d1->id;
@@ -809,10 +832,13 @@ target_stancoa (proposal_fmt * proposal)
             {
                 children (nn, &d1, &d2);
                 if (d1 == oldnn)
-                    d1 = d2;
+		  {
+		    d1 = d2;
+		    d2 = oldnn;
+		  }
 #ifdef BEAGLE
 		id0  = nn->id;
-		id1  = oldnn->id;
+		id1  = new_id(oldnn->id,proposal->world->sumtips);
 		bid1 = oldnn->bid;
 		v1   = oldnn->v;
 		id2  = d1->id;
@@ -865,14 +891,63 @@ t_s_upper (proposal_fmt * proposal, node * connect, node * obb)
     nn = showtop (crawlback (proposal->target));
     oldnn = proposal->target;
     proposal->vs = fabs (nn->tyme - proposal->time);
-    while (nn != connect)
-    {
-        children (nn, &d1, &d2);
-        if (d1 == oldnn)
-            d1 = d2;
+#ifdef BEAGLE
+    children (nn, &d1, &d2);
+    if (d1 == oldnn)
+      {
+	d1 = d2;
+	d2 = oldnn;
+      }
+    id0  = nn->id;
+    id1  = new_id(proposal->oback->id, proposal->world->sumtips);
+    bid1 = proposal->oback->bid;
+    v1   = proposal->vs;
+    id2  = d1->id;
+    bid2 = d1->bid;
+    v2   = d1->v;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+    proposal->vs = nn->v;
+    if (!(nn != connect && nn->type!='r'))
+      {
+    	nn = showtop (crawlback (nn));
+      }
+    else
+      {
+#endif  
+	while (nn != connect && nn->type!='r')
+	  {
+	    children (nn, &d1, &d2);
+	    if (d1 == oldnn)
+	      {
+		d1 = d2;
+		d2 = oldnn;
+	      }
+#ifdef BEAGLE
+	    id0  = nn->id;
+	    id1  = oldnn->id;
+	    bid1 = oldnn->bid;
+	    v1   = proposal->vs;
+	    id2  = d1->id;
+	    bid2 = d1->bid;
+	    v2   = d1->v;
+	    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
+	    pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs, d1->x,
+			  d1->scale, d1->v);
+#endif
+	    proposal->vs = nn->v;
+	    oldnn = nn;
+	    nn = showtop (crawlback (nn));
+	  }
+	children (nn, &d1, &d2);
+	if (d1 == oldnn)
+	  {
+	    d1 = d2;
+	    d2 = oldnn;
+	  }
 #ifdef BEAGLE
 	id0  = nn->id;
-	id1  = oldnn->id;
+	id1  = new_id(oldnn->id, proposal->world->sumtips);
 	bid1 = oldnn->bid;
 	v1   = proposal->vs;
 	id2  = d1->id;
@@ -880,67 +955,51 @@ t_s_upper (proposal_fmt * proposal, node * connect, node * obb)
 	v2   = d1->v;
 	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
 #else
-        pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs, d1->x,
-                      d1->scale, d1->v);
+	pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs, d1->x,
+		      d1->scale, d1->v);
 #endif
-        proposal->vs = nn->v;
-        oldnn = nn;
-        nn = showtop (crawlback (nn));
-    }
-    children (nn, &d1, &d2);
-    if (d1 == oldnn)
-        d1 = d2;
+	proposal->v = obb->tyme - proposal->osister->tyme;
+	proposal->vs = obb->tyme - proposal->ocousin->tyme;
+	if (connect == proposal->ocousin)
+	  {
 #ifdef BEAGLE
-    id0  = nn->id;
-    id1  = oldnn->id;
-    bid1 = oldnn->bid;
-    v1   = proposal->vs;
-    id2  = d1->id;
-    bid2 = d1->bid;
-    v2   = d1->v;
-    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+	    id0  = obb->id;
+	    id1  = proposal->osister->id;
+	    bid1 = proposal->osister->bid;
+	    v1   = proposal->v;
+	    id2  = proposal->ocousin->id;
+	    bid2 = proposal->ocousin->bid;
+	    v2   = proposal->vs;
+	    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
 #else
-    pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs, d1->x,
-                  d1->scale, d1->v);
+	    pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs,
+			  proposal->osister->x, proposal->osister->scale,
+			  proposal->v);
 #endif
-    proposal->v = obb->tyme - proposal->osister->tyme;
-    proposal->vs = obb->tyme - proposal->ocousin->tyme;
-    if (connect == proposal->ocousin)
-      {
+	  }
+	else
+	  {
 #ifdef BEAGLE
-	id0  = obb->id;
-	id1  = proposal->osister->id;
-	bid1 = proposal->osister->bid;
-	v1   = proposal->v;
-	id2  = proposal->ocousin->id;
-	bid2 = proposal->ocousin->bid;
-	v2   = proposal->vs;
-	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+	    id0  = obb->id;
+	    id1  = proposal->osister->id;
+	    bid1 = proposal->osister->bid;
+	    v1   = proposal->v;
+	    id2  = proposal->ocousin->id;
+	    bid2 = proposal->ocousin->bid;
+	    v2   = proposal->vs;
+	    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
 #else
-        pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs,
-                      proposal->osister->x, proposal->osister->scale,
-                      proposal->v);
+	    pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
+			  proposal->ocousin->x, proposal->ocousin->scale,
+			  proposal->vs);
 #endif
+	  }
+#ifdef BEAGLE
       }
-    else
-      {
-#ifdef BEAGLE
-	id0  = obb->id;
-	id1  = proposal->osister->id;
-	bid1 = proposal->osister->bid;
-	v1   = proposal->v;
-	id2  = proposal->ocousin->id;
-	bid2 = proposal->ocousin->bid;
-	v2   = proposal->vs;
-	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+    evaluate_beagle_instances_proposal (proposal, showtop (crawlback (obb)), obb, obb->id, obb->bid,  obb->v);
 #else
-        pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
-                      proposal->ocousin->x, proposal->ocousin->scale,
-                      proposal->vs);
+    pseudoevaluate(proposal,proposal->xf,proposal->mf,showtop (crawlback (obb)), obb, obb->v );
 #endif
-      }
-    pseudoevaluate (proposal, proposal->xf, proposal->mf,
-                    showtop (crawlback (obb)), obb, obb->v);
 }
 
 node *
@@ -973,10 +1032,13 @@ t_s_obbncon (proposal_fmt * proposal, node * obb)
     {
         children (nn, &d1, &d2);
         if (d1 == oldnn)
-            d1 = d2;
+	  {
+	    d1 = d2;
+	    d2 = oldnn;
+	  }
 #ifdef BEAGLE
-	id0  = nn->id;
-	id1  = oldnn->id;
+	id0  = nn->id; // will be offset in prepare_beagle_instances_proposal()
+	id1  = new_id(oldnn->id, proposal->world->sumtips);
 	bid1 = oldnn->bid;
 	v1   = proposal->vs;
 	id2  = d1->id;
@@ -1002,7 +1064,7 @@ t_s_tncon (proposal_fmt * proposal, node * obb)
     long id0, id1, id2;
     long bid1, bid2;
     double v1, v2;
-    id0  = proposal->oback->id;
+    id0  = proposal->oback->id;//id0 will point to the new id+nodep_boundary
     id1  = proposal->origin->id;
     bid1 = proposal->origin->bid;
     v1   = proposal->v;
@@ -1017,54 +1079,83 @@ t_s_tncon (proposal_fmt * proposal, node * obb)
                   proposal->time - proposal->target->tyme);
 #endif
     nn = showtop (crawlback (proposal->target));
-#ifdef BEAGLE
-    oldnn = proposal->oback;
-#else
     oldnn = proposal->target;    
-#endif
     // calculates the time between the newly inserted oback and the back of target
     proposal->v = fabs (nn->tyme - proposal->time);
-    while (nn != proposal->connect)
-      {
-        children (nn, &d1, &d2);
-        if (d1 == oldnn)
-	  d1 = d2;
 #ifdef BEAGLE
-	id0  = nn->id;
-	id1  = oldnn->id;
-	bid1 = oldnn->bid;
-	v1   = oldnn->v;
-	id2  = d1->id;
-	bid2 = d2->bid;
-	v2   = d2->v;
-	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
-#else
-        pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, d1->x, d1->scale, d1->v);
-#endif
-        proposal->v = nn->v;
-        oldnn = nn;
-        nn = showtop (crawlback (nn));
-      }
-    if (obb != proposal->connect)
+    children (nn, &d1, &d2);
+    if (d1 == oldnn)
       {
-#ifdef BEAGLE
-	id0  = nn->id;
-	id1  = oldnn->id;
-	bid1 = oldnn->bid;
-	v1   = proposal->v;
-	id2  = obb->id;
-	bid2 = obb->bid;
-	v2   = obb->v;
-	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
-#else
-        pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
-                      proposal->xt, proposal->mt, proposal->vs);
-#endif
+	d1 = d2;
       }
+    id0  = nn->id;
+    id1  = new_id(proposal->oback->id,proposal->world->sumtips);
+    bid1 = proposal->oback->bid;
+    v1   = proposal->v;
+    id2  = d1->id;
+    bid2 = d1->bid;
+    v2   = d1->v;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+    proposal->v = nn->v;
+    oldnn = nn;
+    if(nn != proposal->connect)
+      {
+	nn = showtop (crawlback (nn));
+#endif
+	while (nn != proposal->connect)
+	  {
+	    children (nn, &d1, &d2);
+	    if (d1 == oldnn)
+	      {
+		d1 = d2;
+		d2 = oldnn;
+	      }
+#ifdef BEAGLE
+	    id0  = nn->id;
+	    id1  = new_id(oldnn->id,proposal->world->sumtips);
+	    bid1 = oldnn->bid;
+	    v1   = oldnn->v;
+	    id2  = d1->id;
+	    bid2 = d1->bid;
+	    v2   = d1->v;
+	    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
+	    pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, d1->x, d1->scale, d1->v);
+#endif
+	    proposal->v = nn->v;
+	    oldnn = nn;
+	    nn = showtop (crawlback (nn));
+	  }
+	if (obb != proposal->connect)
+	  {
+#ifdef BEAGLE
+	    id0  = nn->id;
+	    id1  = new_id(oldnn->id, proposal->world->sumtips);
+	    bid1 = oldnn->bid;
+	    v1   = proposal->v;
+	    id2  = new_id(obb->id, proposal->world->sumtips);
+	    bid2 = obb->bid;
+	    v2   = obb->v;
+	    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
+	    pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
+			  proposal->xt, proposal->mt, proposal->vs);
+#endif
+	  }
+#ifdef BEAGLE
+	}
+#endif
     proposal->v = proposal->connect->v;
+#ifdef BEAGLE
+	evaluate_beagle_instances_proposal (proposal,
+					showtop (crawlback (proposal->connect)), proposal->connect, 
+					    proposal->connect->id, proposal->connect->bid,
+					    proposal->v);
+#else
     pseudoevaluate (proposal, proposal->xf, proposal->mf,
                     showtop (crawlback (proposal->connect)), proposal->connect,
                     proposal->v);
+#endif
 }
 
 void
@@ -1073,14 +1164,17 @@ t_s_tcon (proposal_fmt * proposal, node * uppern)
     node *obb, *d1, *d2;
     children (proposal->target, &d1, &d2);
     if (d1 == uppern)
-        d1 = d2;
+      {
+	d1 = d2;
+	d2 = uppern;
+      }
     proposal->vs = fabs (proposal->target->tyme - showtop (uppern)->tyme);
 #ifdef BEAGLE
     long id0, id1, id2;
     long bid1, bid2;
     double v1, v2;
     id0  = proposal->target->id;
-    id1  = uppern->id;
+    id1  = new_id(uppern->id, proposal->world->sumtips);
     bid1 = uppern->bid;
     v1   = proposal->vs;
     id2  = d1->id;
@@ -1111,22 +1205,28 @@ t_s_tcon (proposal_fmt * proposal, node * uppern)
     {
         children (obb, &d1, &d2);
         if (d1 == proposal->target)
-            d1 = d2;
+	  {
+	    d1 = d2;
+	    d2 = proposal->target;
+	  }
 #ifdef BEAGLE
 	id0  = obb->id;
-	id1  = proposal->oback->id;
+	id1  = new_id(proposal->oback->id, proposal->world->sumtips);
 	bid1 = proposal->oback->bid;
 	v1   = proposal->v;
 	id2  = d1->id;
 	bid2 = d1->bid;
 	v2   = d1->v;
 	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+	evaluate_beagle_instances_proposal (proposal,
+					showtop (crawlback (obb)), obb, obb->id, obb->bid,
+					showtop (crawlback (obb))->tyme - obb->tyme);
 #else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, d1->x,
                       d1->scale, d1->v);
-#endif
         pseudoevaluate (proposal, proposal->xf, proposal->mf,
                         showtop (crawlback (obb)), obb, obb->v);
+#endif
     }
 }
 
@@ -1449,13 +1549,16 @@ pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, MYREAL *lx,
     {
         children (mother, &d1, &d2);
         if (d1 == newdaughter)
-            d1 = d2;
+	  {
+	    d1 = d2;
+	    d2 = newdaughter;
+	  }
 #ifdef BEAGLE
 	long id0, id1, id2;
 	long bid1, bid2;
 	double v1, v2;
 	id0  = mother->id; 
-	id1  = proposal->oback->id;
+	id1  = new_id(proposal->oback->id,proposal->world->sumtips);
 	bid1 = proposal->oback->bid;
 	v1 = v;
 	id2  = d1->id;
@@ -1471,10 +1574,13 @@ pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, MYREAL *lx,
         {
             children (nn, &d1, &d2);
             if (d1 == oldnn)
-                d1 = d2;
+	      {
+		d1 = d2;
+		d2 = oldnn;
+	      }
 #ifdef BEAGLE
 	    id0  = nn->id; 
-	    id1  = d2->id;
+	    id1  = new_id(d2->id,proposal->world->sumtips);
 	    bid1 = d2->bid;
 	    v1 = d2->v;
 	    id2  = d1->id;
