@@ -511,7 +511,7 @@ void read_event_values_fromfile(FILE *file, world_fmt *world)
 	    {
 	      eventbinsize = 2 * age;
 	    }
-	  eventbinnum[pop] = i;
+	  eventbinnum[pop] = i+1;
 	  if(allocsize[spacer + pop] < eventbinnum[pop])
 	    {
 	      allocsize[spacer + pop] += SOME_ELEMENTS;
@@ -737,6 +737,7 @@ void print_event_values(world_fmt * world)
 	else
 	  {
 	    read_event_values_fromfile(world->mighistfile,world);
+	    prepare_event_values(world,sums0, sums1);
 	  }
         // print title to screen
         print_event_values_title(stdout, world->options->progress);
