@@ -16,7 +16,7 @@
  
   Copyright 2002 Peter Beerli and Joseph Felsenstein, Seattle WA
   Copyright 2003 Peter Beerli, Tallahassee FL
-  $Id: migration.h 1711 2010-08-13 19:42:33Z beerli $
+  $Id: migration.h 1737 2010-10-21 16:10:07Z beerli $
   *----------------------------------------------------------------
 */
 /**
@@ -31,6 +31,9 @@
 #include "altivec.h"
 #endif /*ALTIVEC*/
 
+#ifdef ZNZ
+#include "znzlib.h"
+#endif
 
 typedef char allele_type[DEFAULT_ALLELENMLENGTH]; //!< allele type string
 
@@ -346,6 +349,11 @@ typedef struct _bayes
   MYREAL *scaling_factors;
   MYREAL *histtotal;
   MYREAL maxmaxvala;
+  // speed improvement to beat the slow malloc for large string allocation
+  // measures size of first string and uses that currently size * 2;
+  boolean has_linesize;
+  long linesize;
+  long progresslinesize;
 }
 bayes_fmt;
 
@@ -459,7 +467,7 @@ typedef struct _option
   char *datefilename;
   char *bayesfilename;
   char *bayesmdimfilename;
-
+  int use_compressed;
   boolean prioralone;
 
     char title[LINESIZE + 1];
@@ -1010,6 +1018,9 @@ typedef struct _mpirequest_fmt
 } mpirequest_fmt;
 #endif
 /// holds all the parameters etc to run the program
+
+//typedef struct proposal_fmt;
+
 typedef struct _world
 {
   /* generalities */
@@ -1087,6 +1098,9 @@ typedef struct _world
     MYREAL heat;
     MYREAL averageheat;
   MYREAL varheat;
+  long heatid;
+  struct _proposal_fmt *proposal;
+  boolean has_proposal_details;
   MYREAL essminimum;
   MYREAL logprior;
     long treeswapcount;
@@ -1142,7 +1156,11 @@ typedef struct _world
   long bayesaccept;
   bayes_fmt *bayes;
   FILE *bayesfile;
+#ifdef ZNZ
+  znzFile bayesmdimfile;
+#else
   FILE *bayesmdimfile;
+#endif
   FILE *outfile;
   FILE *pdfoutfile;
   FILE *treefile;
@@ -1190,6 +1208,9 @@ typedef struct _world
   long numtimings; // number of elements in the timings vector
   MYREAL **timemodifiers; // vector of parameter modifier vectors
 #endif
+  char * warning;
+  long warningsize;
+  long warningallocsize;
 }
 world_fmt;
 
@@ -1292,12 +1313,8 @@ typedef  struct _proposal_fmt
     MYREAL *param0;
     node *root;
     short migration_model;
-
     long listsize; // holds the size of several nodelists
-
-    
     boolean mig_removed;
-
     MYREAL rr;
     node **nodedata;      // holds abovenodes and bordernodes
     node *origin;            // pointers into nodedata
@@ -1326,7 +1343,6 @@ typedef  struct _proposal_fmt
     MYREAL *mt;
 //could be used to calculate p(gn|go):   MYREAL nu;
 #ifdef UEP
-
     ueparray_fmt ut;
     ueparray_fmt uf;
     node *firstuep;
