@@ -16,7 +16,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: profile.c 1702 2010-06-14 14:18:13Z beerli $
+$Id: profile.c 1722 2010-10-09 15:27:25Z beerli $
  
 -------------------------------------------------------*/
 /*! \file profile.c
@@ -900,7 +900,7 @@ prognose_profile_end (time_t starttime, long numpop2, long nn)
     time_t endtime;
     time_t nowbin;
     struct tm *nowstruct;
-    nowstr = (char *) mycalloc(LINESIZE,sizeof(char));
+    nowstr = (char *) mycalloc(STRSIZE,sizeof(char));
     
     if (tally % nn == 0)
     {
@@ -917,7 +917,7 @@ prognose_profile_end (time_t starttime, long numpop2, long nn)
 
         nowstruct = localtime (&nowbin);
 
-        strftime (nowstr, LINESIZE, "%H:%M %B %d %Y", nowstruct);
+        strftime (nowstr, STRSIZE, "%H:%M %B %d %Y", nowstruct);
         FPRINTF (stdout, "           Prognosed end of run: %s\n", nowstr);
     }
     else
